local mod = RW:Boss("The Lich King", "Icecrown Citadel", "The Frozen Throne")
local L

mod.BossIcon = [[Interface\Icons\achievement_boss_lichking]]
mod.SubCatIcon = [[Interface\Icons\achievement_dungeon_frozenthrone]]

------------------------------------
-- Variables
local mPhase, tActive

-- Shambling Horror
local tShamblingCD, wShambling, tEnrageCD, tEnrageCast, wEnrage

-- Infest
local tInfestCD, tInfestCast, wInfest

-- Necrotic Plague
local tNecroticCD, wNecrotic, mNecrotic, iNecrotic

-- Soul Reaper
local tReaperCD, wReaper, mReaper

-- Defile
local tDefileCD, wDefileCast, wDefileRun, iDefile, mDefile

-- Valkyr
local tValkyrCD, wValkyr, iValkyr

-- Remorseless Winter
local tRemorselessDur, wRemorseless

-- Raging Spirits
local mRagingSpirit, iRagingSpirit, wRagingSpirit

-- Vile Spirits
local tSpiritsCD, tSpiritsAttack, wSpirits

-- Harvest Sooul
local tHarvestCD, tHarvestDur, wHarvest, mHarvest

-----------------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale("enUS")
	if L then
		L.YellTrigger = "So the Light's vaunted justice has finally arrived? Shall I lay down Frostmourne and throw myself at your mercy, Fordring?"
		
		L.defile_on_player = "Defile on me!"
		
		L.active = " active"
		L.active_opt = "Timer for when the fight starts"
		L.active_grup = "Fight start"
		
		L.spirits_attack = "Spirits Attacking"
		L.spirits_attack_opt = "Timer for when the Vile spirits will attack"
		
		L.plague_hisper = "You have got the plague!!"
		L.plague_opt = "Whisper people that get Necrotic Plague"
	end
	L = mod:GetLocale()
	mod.L = L
	mod:SetTrigger(36597, L.YellTrigger)
	mod:SetWin()
	mod:SetDelay(60)
	
	self:AddToCategory("Phase 1", 70372, 70337, 72143)
	self:AddToCategory("Phase 2", 69409, 72762, 71843)
	self:AddToCategory("Phase 3", 68980, 70498)
	self:AddToCategory("Phase Transitions", 74270, 69200)
	
	self:AddBoolOption("WhisperOnPlague", L.plague_opt, nil, false, select(3, GetSpellInfo(70337)))
	
	mPhase 				= self:PhaseMessage()
	tActive				= self:CustomTimer(55, (RW.LBB[self:GetName()] or self:GetName())..L.active, L.active_opt, [[Interface\Icons\achievement_boss_lichking]],  L.active_grup)
	
	-- Shambling Horror
	tShamblingCD		= self:CDTimer(60, 70372)
	tEnrageCD			= self:CDTimer(20, 72143, nil, nil, false)
	tEnrageCast			= self:CastTimer(1, 72143, nil, nil, false)
	wShambling			= self:Warning(70372)
	wEnrage				= self:CastWarning(72143, nil, nil, false)
	
	-- Infest
	tInfestCD			= self:CDTimer(20, 70541)
	tInfestCast			= self:CastTimer(70541)
	wInfest				= self:Warning(70541)
	
	-- Necrotic Plague
	tNecroticCD			= self:CDTimer(30, 70337)
	mNecrotic			= self:TargetMessage(70337)
	wNecrotic			= self:RunawayWarning(70337)
	iNecrotic			= self:Icon("CROSS", 70337, false, 10, "SQUARE")
	
	-- Soul Reaper
	tReaperCD			= self:CDTimer(30, 69409)
	wReaper				= self:InformWarning(69409)
	mReaper				= self:TargetMessage(69409)
	
	-- Defile
	tDefileCD			= self:CDTimer(30, 72762)
	wDefileCast			= self:CastWarning(72762)
	wDefileRun			= self:RunawayWarning(72762)
	iDefile				= self:Icon("SKULL", 72762, false, 6)
	mDefile				= self:TargetMessage(72762)
	
	-- Valkyr
	tValkyrCD			= self:CDTimer(47, 71843)
	wValkyr				= self:Warning(71843)
	iValkyr				= self:Icon("TRIANGLE", 71843, false, nil, "MOON", "DIAMOND")
	
	-- Remorseless Winter
	tRemorselessDur		= self:DurTimer(60, 74270)
	wRemorseless		= self:RunawayWarning(74270)
	
	-- Raging Spirits
	mRagingSpirit		= self:TargetMessage(69200)
	wRagingSpirit		= self:OnYouWarning(69200)
	iRagingSpirit		= self:Icon("TRIANGLE", 69200, false, 5)
	
	-- Vile Spirits
	tSpiritsCD			= self:CDTimer(32, 70498)
	tSpiritsAttack		= self:CustomTimer(15, L.spirits_attack, L.spirits_attack_opt, 70498)
	wSpirits			= self:CastWarning(70498)
	
	-- Harvest Soul
	tHarvestCD			= self:CDTimer(75, 68980)
	tHarvestDur			= self:DurTargetTimer(6, 68980)
	wHarvest			= self:CastWarning(68980)
	mHarvest			= self:TargetMessage(68980)
end

-----------------------------------------
-- Locals
local phase
local defileSpamm
local necrotic = GetSpellInfo(70337)
local valkyrSpam
local markedValkyrs = {}
local plagueSpam = {}

-----------------------------------------
-- Start

function mod:OnEngage(trigger)
	self:Track("ShamblingHorror", 70372)
	self:Track("ShamblingEnrage", 72143)
	self:Track("Infest", 70541, 73779, 73780, 73781)
	self:Track("NercroticPlague", 70337, 70338, 73785, 73786, 73787, 73912, 73913, 73914)
	self:Track("SoulReaper", 69409, 73797)
	self:Track("Defile", 72762, 72754, 73708, 73709, 73710) -- First is the spell casted by arthas to summon the Defile
	self:Track("Remorseless", 74270, 68981, 72259, 74271, 74272, 74273, 74274, 74275)
	self:Track("Valkyr", 69037)
	self:Track("RagingSpirit", 69200)
	self:Track("Spirits", 70498)
	self:Track("FuryOfFrostmourn", 72350, 70063, 72351)
	self:Track("HarvestSoul", 68980, 74325, 74326, 44327)
	
	self:RegisterEvent("UNIT_AURA")
	
	if  trigger == L.YellTrigger then
		tActive:Start()
		tNecroticCD:Schedule(55)
	end
	
	wipe(plagueSpam)
	wipe(markedValkyrs)
	valkyrSpam = 0
	defileSpamm = 0
	phase = 1
end

function mod:UNIT_AURA(event, uID)
	local debuffed = UnitDebuff(uID, necrotic)
	local name = UnitName(uID)
	if debuffed then
		if not plagueSpam[name] then
			plagueSpam[name] = true
		
			if name == self.pName then
				wNecrotic:Show()
				
			elseif self.Party.group[name] then
				mNecrotic:Show(name)
				if self.db.profile.WhisperOnPlague then
					SendChatMessage(L.plague_hisper, "WHISPER", nil, name)
				end
			end
			
			iNecrotic:Show(uID)			
		end
	else
		plagueSpam[name] = nil
	end
end

function mod:ShamblingHorror(event, args)
	if event == "SPELL_SUMMON" then
		tShamblingCD:Start()
		wShambling:Show()
	end
end

function mod:ShamblingEnrage(event, args)
	if event == "SPELL_CAST_START" then
		tEnrageCD:Start()
		tEnrageCast:Start()
		wEnrage:Start()
	end
end

function mod:Infest(event, args)
	if event == "SPELL_CAST_START" then
		tInfestCD:Start()
		tInfestCast:Start()
		wInfest:Show()
	end
end

function mod:NercroticPlague(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		tNecroticCD:Start()
	
	elseif event == "SPELL_DISPELL" then
		iNecrotic:Hide(args.dName)
	end
end

function mod:SoulReaper(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		tReaperCD:Start()
	
	elseif event == "SPELL_AURA_APPLIED" then
		if args:IsPlayer() then
			wReaper:Show()
		end
		mReaper:Show(args.dName)
	end
end

local function ScanDefileTarget()
	local target = mod:GetMobTarget(36597)
	if target then
		iDefile:Show(target)
		mDefile:Show(target)
	
		if target == mod.pName then
			SendChatMessage(L.defile_on_player, "SAY")
			wDefileRun:Show()
			
		elseif mod.Party.group[target] then
			local uID = mod.Party.group[target].id
			if uID and CheckInteractDistance(uID, 2) then
				wDefileRun:Show()
			end
		end
	end
end

function mod:Defile(event, args)
	if event == "SPELL_CAST_START" then
		tDefileCD:Start()
		wDefileCast:Show()
		self:Schedule("DefileScan", 0.1, ScanDefileTarget)
		
	elseif event == "SPELL_DAMAGE" then
		if args:IsPlayer() and GetTime() - defileSpamm > 2 then
			wDefileRun:Show()
			defileSpamm = GetTime()
		end
	end
end

local function unregisterUnitAura()
	mod:UnregisterEvent("UNIT_AURA")
end

function mod:Remorseless(event, args)
	if event == "SPELL_CAST_START" then
		wRemorseless:Show()
		tRemorselessDur:Start()
		tReaperCD:Schedule(60, 39)
		
		phase = phase + 1
		mPhase:Schedule(60, phase)
		if phase == 2 then
			tValkyrCD:Schedule(60, 20)
			self:Schedule("UnregisterUnitAura", 60, unregisterUnitAura)
		elseif phase == 3 then
			tHarvestCD:Schedule(60, 15)
		end
	end
end

local function valkyrScanner()
	for name, data in mod.Party:PartyIterator() do
		local cID = mod:CUIDFromUID(data.id.."target")
		if cID == 36609 then
			local GUID = UnitGUID(data.id.."target")
			if not markedValkyrs[GUID] and GetRaidTargetIndex(data.id.."target") == 0 then
				iValkyr:Show(data.id.."target")
				markedValkyrs[GUID] = true
			end
		end
	end
	
	if GetTime() - valkyrSpam < 5 then
		mod:Schedule("ValkyrScan", 0.2, valkyrScanner)
	end
end

function mod:UPDATE_MOUSEOVER_UNIT()
	local cID = mod:CUIDFromUID("mouseover")
	if cID == 36609 then
		local GUID = UnitGUID("mouseover")
		if not markedValkyrs[GUID] and GetRaidTargetIndex("mouseover") == 0 then
			iValkyr:Show("mouseover")
			markedValkyrs[GUID] = true
		end
	end

	if GetTime() - valkyrSpam > 5 then
		self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
	end
end

function mod:Valkyr(event, args)
	if event == "SPELL_SUMMON" and (GetTime() - valkyrSpam > 2) then
		wValkyr:Show()
		tValkyrCD:Start()
		self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
		valkyrSpam = GetTime()
		self:Schedule("ValkyrScan", 0.5, valkyrScanner)
	end
end

function mod:RagingSpirit(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		if args:IsPlayer() then
			wRagingSpirit:Show()
		end
		mRagingSpirit:Show(args.dName)
		iRagingSpirit:Show(args.dName)
	end
end

function mod:Spirits(event, args)
	if event == "SPELL_CAST_START" then
		tSpiritsCD:Start()
		tSpiritsAttack:Start()
		wSpirits:Show()
	end
end

local function releaseHold()
	mod:ToggleHold()
end

function mod:FuryOfFrostmourn(event, args)
	if event == "SPELL_CAST_START" then
		self:ToggleHold()
		self:Schedule("ReleaseHold", 180, releaseHold)
	end
end

function mod:HarvestSoul(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		tHarvestCD:Start()
		tHarvestDur:Start(args.dName)
		mHarvest:Show(args.dName)
		wHarvest:Show()
	end
end